#!/usr/bin/env python

import numpy as np
import matplotlib.pyplot as plt
import wuml

class histograms:
	def __init__(self, x, num_bins=10, title='', fontsize=12, facecolor='blue', α=0.5, 
			xlabel='', ylabel='', ticker_fontsize=9, ticker_rotate=0,
			path=None, subplot=None, ylogScale=False, show=True, normalize=False, 
			title_font=12, xfont=12, yfont=12, figsize=None):

		x = wuml.ensure_numpy(x)
		self.title_font = title_font
		self.xfont = xfont
		self.yfont = yfont
		self.already_called_plot_line = False
		if figsize is not None: plt.figure(figsize=figsize)

		self.histogram(x=x, num_bins=num_bins, title=title, fontsize=fontsize, facecolor=facecolor, 
				α=α, xlabel=xlabel, ylabel=ylabel, ticker_fontsize=ticker_fontsize, ticker_rotate=ticker_rotate, 
				path=path, subplot=subplot, ylogScale=ylogScale, show=show, normalize=normalize)

	def histogram(self, x, num_bins=10, title='', fontsize=12, facecolor='blue', α=0.5, 
			xlabel='', ylabel='', ticker_fontsize=9, ticker_rotate=0,
			path=None, subplot=None, ylogScale=False, show=True, normalize=False):
		'''
			normalize 	: Show the probability instead of count
			show		: Calls the show function at the end
		'''
		x = wuml.ensure_numpy(x)
		if subplot is not None: plt.subplot(subplot)


		plt.tick_params(axis='both', which='both', labelsize=ticker_fontsize)
		plt.xticks(fontsize=ticker_fontsize, rotation=ticker_rotate)
		#plt.yticks(fontsize=ticker_fontsize, rotation=ticker_rotate)

		plt.xlabel(xlabel, fontsize=fontsize)
		plt.ylabel(ylabel, fontsize=fontsize)
		plt.title(title, fontsize=fontsize)

		n, bins, patches = plt.hist(x, num_bins, facecolor=facecolor, alpha=α, density=normalize)
		if ylogScale: plt.yscale('log', nonpositive='clip')
		plt.tight_layout()

		#hist, bins = np.histogram(x, bins=num_bins)
		#logbins = np.logspace(np.log10(bins[0]),np.log10(bins[-1]),len(bins))
		#plt.hist(x, bins=logbins)
		#plt.yscale('log')

		#logbins = np.logspace(np.log10(bins[0]),np.log10(bins[-1]),len(bins))
		#plt.hist(x, bins=logbins)
		#plt.xscale('log')
	
		if subplot is None:
			if path is not None: plt.savefig(path)
			if show: plt.show()

	def show(self, save_path=None):
		if save_path is None: plt.show()
		else: plt.savefig(save_path)




#!/usr/bin/env python

import wuml
import numpy as np
import matplotlib.pyplot as plt
import sklearn.metrics
from sklearn.cluster import KMeans
from sklearn.cluster import SpectralClustering
from sklearn.preprocessing import normalize			# version : 0.17
import matplotlib.pyplot as plt


class plot_clusters:
	def __init__(self, X, allocation, title='', xlabel='', ylabel='', 
					figsize=None, fsize=14, useGrid=True, subplot=None, imgText=None, 
					xTextShift=0.05, yTextShift=0.95, xTextLoc=None, yTextLoc=None):
		X = wuml.ensure_numpy(X)

		cmap = ['b', 'g', 'r', 'c', 'm', 'y','k']
		labels = np.unique(allocation)
		n = labels.shape[0]
	
		if figsize is not None: plt.figure(figsize=figsize) # figsize=(3,2)
		#f, axs = plt.subplots(2,2,figsize=(15,15))

		plt.style.use('default')
		#plt.style.use('seaborn')
		#plt.style.use("ggplot")
		#plt.rcParams['axes.edgecolor'] = "#777777"
		#plt.rcParams['axes.facecolor'] = '#FFFFFF'
	
		#ax = plt.axes()
		#plt.axis([xmin, xmax, ymin, ymax])
		#ax.set_aspect('equal')
		#ax.grid(True, which='both')
	
		#ax.set(facecolor = 'white')
		#ax.axhline(y=0, color='k')
		#ax.axvline(x=0, color='k')
	
		if subplot is not None: plt.subplot(subplot)
		for i, j in enumerate(labels):
			subX = X[allocation == labels[i]]
			plt.plot(subX[:,0], subX[:,1], cmap[i] + '.')
	
		plt.xlabel(xlabel)
		plt.ylabel(ylabel)
		plt.title(title, fontsize=fsize)
		self.add_text(X[:,0], X[:,1], imgText, α=xTextShift, β=yTextShift, xTextLoc=xTextLoc, yTextLoc=yTextLoc)
		if useGrid: plt.grid(linestyle='dotted')
		if subplot is None: 
			plt.tight_layout()
			plt.show()

	def add_text(self, X, Y, textstr, α=0.05, β=0.95, xTextLoc=None, yTextLoc=None):
		if textstr is None: return
		mX = np.min(X)
		mY = np.min(Y)

		if xTextLoc is None: xLoc = α*(np.max(X) - mX) + mX
		else: xLoc = xTextLoc

		if yTextLoc is None: yLoc = β*(np.max(Y) - mY) + mY
		else: yLoc = yTextLoc

		props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
		plt.text(xLoc, yLoc, textstr, fontsize=14, verticalalignment='top', bbox=props)


	def show(self):
		#plt.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=None, hspace=None)
		#plt.subplots_adjust(hspace=1.0, wspace=1.0)
		plt.tight_layout()
		plt.show()

if __name__ == "__main__":
	n = 100
	x1 = np.random.randn(n,2) + np.array([4,4])
	x2 = np.random.randn(n,2) + np.array([-4,-4])
	X = np.vstack((x1,x2))
	
	Y = np.concatenate([np.zeros(n), np.ones(n)])
	
	cluster_plot(X, Y, title='Clustering Results')



#!/usr/bin/env python

from matplotlib import pyplot as plt
import numpy as np


class scatter:
	"""A class that generates basic scatter plots"""
	def __init__(self, X, Y, title='title', xlabel='xlable', ylabel='ylable', imgText=None, outpath=None, 
					subplot=None, xlim=None, ylim=None, xTextShift=0.05, yTextShift=0.95,
					ticker_fontsize=9, xTextLoc=None, yTextLoc=None, color='blue', marker='x',
					xticker_rotate=0, yticker_rotate=0,
					x_origin_axis_color=None, y_origin_axis_color=None,
					xtick_locations=None, xtick_labels=None, ytick_locations=None, ytick_labels=None, 
					title_font=16, xfont=16, yfont=16, figsize=None, show=True):
		"""calculates the probability that "token" is found in spam emails
		:param token: (str)
		x_origin_axis_color	: 'k' is default black, or 'red'
		y_origin_axis_color
		:return: (float) probability "token" is spam based on training emails
		"""
		self.title_font = title_font
		self.xfont = xfont
		self.yfont = yfont
		self.already_called_plot_line = False
		if figsize is not None: plt.figure(figsize=figsize)

		#plt.grid(True, which='both')
		if x_origin_axis_color is not None:
			plt.axvline(x=0, color=x_origin_axis_color)
		if y_origin_axis_color is not None:
			plt.axhline(y=0, color=x_origin_axis_color)

		self.plot_scatter(X=X, Y=Y, title=title, xlabel=xlabel, ylabel=ylabel, imgText=imgText, outpath=outpath, 
					subplot=subplot, xlim=xlim, ylim=ylim, xTextShift=xTextShift, yTextShift=yTextShift,
					ticker_fontsize=ticker_fontsize, xTextLoc=xTextLoc, yTextLoc=yTextLoc, 
					color=color, marker=marker, xticker_rotate=xticker_rotate, yticker_rotate=yticker_rotate,
					xtick_locations=xtick_locations, xtick_labels=xtick_labels, 
					ytick_locations=ytick_locations, ytick_labels=ytick_labels, show=show)

	def plot_scatter(self, X, Y, title, xlabel, ylabel, imgText=None, outpath=None, 
					subplot=None, xlim=None, ylim=None, xTextShift=0.05, yTextShift=0.95,
					ticker_fontsize=9, xTextLoc=None, yTextLoc=None, color='blue', marker='x',
					xticker_rotate=0, yticker_rotate=0, show=True,
					xtick_locations=None, xtick_labels=None, ytick_locations=None, ytick_labels=None):
		"""create a default 2D line plot
		
		: X: (float) the values for x-axis
		: Y: (float) the values for y-axis
		: xTextShift: (float) Uses the range of x values as total, sets a percentage of place the text
		: yTextShift: (float) Uses the range of y values as total, sets a percentage of place the text
		: xTextLoc: (float) Uses absolute value and overrides xTextShift
		: yTextLoc: (float) Uses absolute value and overrides yTextShift

		#color='blue'        # specify color by name
		#color='g'           # short color code (rgbcmyk)
		#color='0.75'        # Grayscale between 0 and 1
		#color='#FFDD44'     # Hex code (RRGGBB from 00 to FF)
		#color=(1.0,0.2,0.3) # RGB tuple, values 0 to 1
		#color='chartreuse'; # all HTML color names supported

		#Possible markers: https://matplotlib.org/stable/api/markers_api.html

		:return: None
		"""

		self.X = X
		self.Y = Y

		if X is None: X = np.arange(1, len(Y)+1)
		if subplot is not None: plt.subplot(subplot)

		self.add_plot(X,Y, color, marker)
		self.set_title(title, fontsize=self.title_font)
		self.set_xlabel(xlabel, fontsize=self.xfont)
		self.set_ylabel(ylabel, fontsize=self.yfont)
		self.add_text(X, Y, imgText, α=xTextShift, β=yTextShift, xTextLoc=xTextLoc, yTextLoc=yTextLoc)

		#plt.tick_params(axis='both', which='both', labelsize=ticker_fontsize)
		plt.xticks(ticks=xtick_locations, labels=xtick_labels, fontsize=ticker_fontsize, rotation=xticker_rotate)
		plt.yticks(fontsize=ticker_fontsize, rotation=yticker_rotate, ticks=ytick_locations, labels=ytick_labels )

		if xlim is not None: plt.xlim(xlim)
		if ylim is not None: plt.ylim(ylim)
		
		if subplot is None and outpath is None and show: 
			plt.tight_layout()
			plt.show()
		elif outpath is not None: plt.savefig(outpath)

	def add_text(self, X, Y, textstr, α=0.05, β=0.95, xTextLoc=None, yTextLoc=None):
		if textstr is None: return
		mX = np.min(X)
		mY = np.min(Y)

		if xTextLoc is None: xLoc = α*(np.max(X) - mX) + mX
		else: xLoc = xTextLoc

		if yTextLoc is None: yLoc = β*(np.max(Y) - mY) + mY
		else: yLoc = yTextLoc

		props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
		plt.text(xLoc, yLoc, textstr, fontsize=14, verticalalignment='top', bbox=props)



	def set_title(self, title, fontsize=16):
		plt.title(title, fontsize=fontsize)

	def set_xlabel(self, xlabel, fontsize=16):
		plt.xlabel(xlabel, fontsize=fontsize)

	def set_ylabel(self, ylabel, fontsize=16):
		plt.ylabel(ylabel, fontsize=fontsize)

	def add_plot(self, X,Y, color='blue', marker='x'):
		#color='blue'        # specify color by name
		#color='g'           # short color code (rgbcmyk)
		#color='0.75'        # Grayscale between 0 and 1
		#color='#FFDD44'     # Hex code (RRGGBB from 00 to FF)
		#color=(1.0,0.2,0.3) # RGB tuple, values 0 to 1
		#color='chartreuse'; # all HTML color names supported

		#Possible markers: https://matplotlib.org/stable/api/markers_api.html

		self.X = X
		self.Y = Y

		plt.scatter(X, Y, color=color, marker=marker)

	def show(self, save_path=None, title=None, xlabel=None, ylabel=None, imgText=None, 
					xlim=None, ylim=None, xTextShift=0.05, yTextShift=0.95,
					xTextLoc=None, yTextLoc=None):

		if title is not None: self.set_title(title, fontsize=self.title_font)
		if xlabel is not None: self.set_xlabel(xlabel, fontsize=self.xfont)
		if ylabel is not None: self.set_ylabel(ylabel, fontsize=self.yfont)
		if imgText is not None: 
			self.add_text(self.X, self.Y, imgText, α=xTextShift, β=yTextShift, xTextLoc=xTextLoc, yTextLoc=yTextLoc)

		plt.tight_layout()
		if save_path is None: 
			plt.tight_layout()
			plt.show()
		else: plt.savefig(save_path)


	def plot_line_with_error_area(x, y, error, show=False):
		#x = np.linspace(0, 30, 30)
		#y = np.sin(x/6*np.pi)
		#error = np.random.normal(0.1, 0.02, size=y.shape)
		#y += np.random.normal(0, 0.1, size=y.shape)
		
		plt.plot(x, y, 'k-')
		plt.fill_between(x, y-error, y+error)
		if show: 
			plt.tight_layout()
			plt.show()


if __name__ == "__main__":
##	Example 1	
#	textstr = '\n'.join(( r'$\mu=%.2f$' % (0.1, ), r'$\mathrm{median}=%.2f$' % (0, ), r'$\sigma=%.2f$' % (33, )))
#	x = np.linspace(0, 10, 1000)
#	y = np.sin(x)
#
#	lp = lines()
#	lp.plot_line(x, y, 'Title Here', 'X axis', 'Y axis', imgText=textstr)#, outpath)

#	Example 2
	textstr = '\n'.join(( r'$\mu=%.2f$' % (0.1, ), r'$\mathrm{median}=%.2f$' % (0, ), r'$\sigma=%.2f$' % (33, )))
	x = np.array([0,1,2])
	y = np.array([1,2,3])
	textstr = 'X   Y\n-----\n0   1\n1   2\n2   3'
	pt = scatter(x, y, 'Mysterious Data We Want to Model', 'X axis', 'Y axis', xlim=[-2,6], ylim=[-2,6], imgText=textstr, xTextLoc=-1.5, yTextLoc=5.5)



#! /usr/bin/env python


import numpy as np; np.random.seed(0)
import seaborn as sns
from matplotlib import pyplot as plt
import matplotlib
from numpy import genfromtxt
from sklearn.cluster import SpectralClustering
from scipy.sparse import coo_matrix
from sklearn.utils import shuffle


class heatMap:
	def __init__(self, kernel, title='', xlabel='Features', ylabel='Samples', ticker_fontsize=9,
						fsize=14, use_seaborn=False, vmin=0, vmax=1, show=True,
						center=None, linewidths=0, cmap=None, outpath='', subplot=None,
						xticker_rotate=0, yticker_rotate=0, sort_kernel_based_on_label=False, label=None, 
						xtick_locations=None, xtick_labels=None, ytick_locations=None, ytick_labels=None, figsize=None):

		self.plot_font_size = 4
		font = {'family' : 'normal', 'weight' : 'bold', 'size'   : self.plot_font_size}
		matplotlib.rc('font', **font)
		self.cluster_by_id = {}
		self.cluster_by_name = {}

		if figsize is not None: plt.figure(figsize=figsize)


		if sort_kernel_based_on_label:
			kernel = self.sort_kernel(kernel, label)

		self.draw_HeatMap(kernel, title=title, xlabel=xlabel, ylabel=ylabel, ticker_fontsize=ticker_fontsize,
						fsize=fsize, use_seaborn=use_seaborn, vmin=vmin, vmax=vmax, show=show,
						center=center, linewidths=linewidths, cmap=cmap, outpath=outpath, subplot=subplot,
						xticker_rotate=xticker_rotate, yticker_rotate=yticker_rotate,
						xtick_locations=xtick_locations, xtick_labels=xtick_labels, 
						ytick_locations=ytick_locations, ytick_labels=ytick_labels)




	def sort_kernel(self, kernel, allocation, item_labels=[] ):
		alloc_list = np.unique(allocation) 
		ksize = kernel.shape[0]
		self.rearrangement = []

		sorted_kernel = np.empty((0, ksize))
		for m in alloc_list:
			new_list = np.where(allocation == m)[0].tolist()
			self.cluster_by_id[m] = new_list
			self.rearrangement.extend(new_list)

			if len(item_labels) > 0:
				self.cluster_by_name[m] = []
				for q in new_list:
					self.cluster_by_name[m].append( item_labels[q] )


			f = kernel[allocation == m, :]
			sorted_kernel = np.vstack((sorted_kernel, f))
			
		H_sorted_kernel = np.empty((ksize,0))
		for m in alloc_list:
			f = sorted_kernel[:,allocation == m]
			H_sorted_kernel = np.hstack((H_sorted_kernel, f))

		self.sorted_kernel = H_sorted_kernel


		#	Sort the labels if they exist
		if len(item_labels) > 0:
			self.sorted_labels = []
			for m in self.rearrangement:
				self.sorted_labels.append( item_labels[m] )

		return H_sorted_kernel

	def draw_HeatMap(self, kernel, title='', 
						xlabel='Features', ylabel='Samples', ticker_fontsize=9,
						fsize=14, use_seaborn=False, vmin=0, vmax=1, 
						center=None, linewidths=0, cmap=None, outpath=None, subplot=None,
						xticker_rotate=0, yticker_rotate=0, show=False,
						xtick_locations=None, xtick_labels=None, ytick_locations=None, ytick_labels=None):

		if use_seaborn:
			ax = sns.heatmap(kernel, vmin=vmin, vmax=vmax, center=center, linewidths=linewidths, cmap=cmap)
			plt.title(title, fontsize=fsize)
			plt.xticks(fontsize=ticker_fontsize, rotation=xticker_rotate)
			plt.yticks(fontsize=ticker_fontsize, rotation=yticker_rotate)
		
			plt.tight_layout()
			plt.show() 
		else:
			
			kernel = np.flipud(kernel)
			#fig, ax = plt.subplots()
			#plt.subplots(2,1)
			#if subplot is None: fig, ax = plt.subplots()
			#else: fig, ax = plt.subplots(subplot)
			#fig.set_size_inches(13,13)

			if subplot is not None: plt.subplot(subplot)
			heatmap = plt.pcolor(kernel, cmap=matplotlib.cm.Blues, alpha=0.8)

			#yTicklabel = list(reversed(yTicklabel))
			#if len(yTicklabel) > 0:
			#	ax.set_yticks(np.arange(kernel.shape[0]) + 0.5, minor=False)
			#	ax.set_yticklabels(yTicklabel, rotation='horizontal', minor=False, size=fsize)
			
			#if len(xTicklabel) > 0:
			#	ax.set_xticks(np.arange(kernel.shape[1]) + 0.5, minor=False)
			#	ax.set_xticklabels(xTicklabel, rotation='vertical', minor=False, size=fsize)
		 
			plt.title(title, fontsize=fsize)
			plt.xticks(ticks=xtick_locations, labels=xtick_labels, fontsize=ticker_fontsize, rotation=xticker_rotate)
			plt.yticks(fontsize=ticker_fontsize, rotation=yticker_rotate, ticks=ytick_locations, labels=ytick_labels )

			#ax.tick_params(axis='both', which='major', labelsize=ticker_fontsize)
			#ax.tick_params(axis='both', which='minor', labelsize=ticker_fontsize)
			#plt.xticks(fontsize=ticker_fontsize, rotation=ticker_rotate)
			#plt.yticks(fontsize=ticker_fontsize, rotation=ticker_rotate)

			if xlabel != '': plt.xlabel(xlabel, fontsize=fsize)
			if ylabel != '': plt.ylabel(ylabel, fontsize=fsize)
		

			if subplot is None:
				if outpath is not None: plt.savefig(outpath)
				if show: 
					plt.tight_layout()
					plt.show()

		return plt

	def show(self):
		plt.tight_layout()
		plt.show()

if __name__ == "__main__":
	#	Heat Map ticker
	K = np.array([[0.7, 0.7, 0, 0],[0.5,0.5,0,0],[0,0,0.4, 0.8], [0,0,0.4, 0.8]])
	hMap = heatMap(K, title='Heat Map', xtick_locations=[0.5, 1.5, 2.5, 3.5], xtick_labels=['A', 'B', 'C','D'],
						ytick_locations=[0.5, 1.5, 2.5, 3.5], ytick_labels=['A', 'B', 'C','D'])
	
	#	Heat Map Example
	X1 = np.random.randn(100,2)
	X2 = np.random.randn(100,2) + 5
	X = np.vstack((X1,X2))
	np.random.shuffle(X)		# randomly shuffle the rows of X
	
	clf = SpectralClustering(n_clusters=2)
	allocation = clf.fit_predict(X)
	kernel = clf.affinity_matrix_
	axis_label = range(kernel.shape[0])
	
	hM = heatMap(kernel, title='Drawing Unsorted Heat Map', sort_kernel_based_on_label=False, subplot=121, figsize=(10,5))
	heatMap(kernel, title='Drawing Sorted Heat Map', sort_kernel_based_on_label=True, label=allocation, subplot=122)
	hM.show()

	#cmap types: "Blues", "YlGnBu", "BuPu", "Greens", None



#!/usr/bin/env python
from matplotlib import pyplot as plt
import numpy as np


class bar:
	"""A class that generates basic bar plots"""
	def __init__(self, X, Y, title='', xlabel='', ylabel='', imgText=None, outpath=None, horizontal=False,
					subplot=None, xlim=None, ylim=None, xTextShift=0.05, yTextShift=0.95,
					ticker_fontsize=9, xTextLoc=None, yTextLoc=None, color='blue', marker=',', show=True, 
					xtick_labels=None, xticker_rotate=0, yticker_rotate=0,
					ytick_locations=None, ytick_labels=None, 
					title_font=16, xfont=16, yfont=16, figsize=None, disable_xticks=False, disable_yticks=False):

		"""calculates the probability that "token" is found in spam emails
		
		:param token: (str)
		:return: (float) probability "token" is spam based on training emails
		"""

		self.title_font = title_font
		self.xfont = xfont
		self.yfont = yfont
		self.already_called_plot_line = False
		plt.style.use('ggplot')
		if figsize is not None: plt.figure(figsize=figsize)
	
		self.plot_bar(X=X, Y=Y, title=title, xlabel=xlabel, ylabel=ylabel, imgText=imgText, outpath=outpath, horizontal=horizontal,
					subplot=subplot, xlim=xlim, ylim=ylim, xTextShift=xTextShift, yTextShift=yTextShift,
					ticker_fontsize=ticker_fontsize, xTextLoc=xTextLoc, yTextLoc=yTextLoc, color=color, marker=marker, show=show, 
					xtick_labels=None, xticker_rotate=xticker_rotate, yticker_rotate=0,
					ytick_locations=None, ytick_labels=None)

		if disable_xticks: self.disable_xticks()
		if disable_yticks: self.disable_yticks()

	def disable_xticks(self):
		plt.xticks([])

	def disable_yticks(self):
		plt.yticks([])

	def plot_bar(self, X, Y, title, xlabel, ylabel, imgText=None, outpath=None, horizontal=False,
					subplot=None, xlim=None, ylim=None, xTextShift=0.05, yTextShift=0.95,
					ticker_fontsize=9, xTextLoc=None, yTextLoc=None, color='blue', marker=',', show=True, 
					xtick_labels=None, xticker_rotate=0, yticker_rotate=0,
					ytick_locations=None, ytick_labels=None):

		if subplot is not None: plt.subplot(subplot)
		x_pos = [i for i, _ in enumerate(X)]
		if horizontal:
			plt.barh(x_pos, Y, color=color)

			self.set_ylabel(xlabel, fontsize=self.xfont)
			self.set_xlabel(ylabel, fontsize=self.yfont)
			if imgText is not None: 
				self.add_text(Y, x_pos, imgText, α=yTextShift, β=xTextShift, xTextLoc=yTextLoc, yTextLoc=xTextLoc)
	
			if xtick_labels is None: plt.yticks(ticks=x_pos, labels=X, fontsize=ticker_fontsize, rotation=xticker_rotate)
			else: plt.yticks(ticks=x_pos, labels=xtick_labels, fontsize=ticker_fontsize, rotation=xticker_rotate)

			plt.xticks(fontsize=ticker_fontsize, rotation=yticker_rotate, ticks=ytick_locations, labels=ytick_labels )

		else:
			plt.bar(x_pos, Y, color=color)

			self.set_xlabel(xlabel, fontsize=self.xfont)
			self.set_ylabel(ylabel, fontsize=self.yfont)
			if imgText is not None: 
				self.add_text(x_pos, Y, imgText, α=xTextShift, β=yTextShift, xTextLoc=xTextLoc, yTextLoc=yTextLoc)

			if xtick_labels is None: plt.xticks(ticks=x_pos, labels=X, fontsize=ticker_fontsize, rotation=xticker_rotate)
			else: plt.xticks(ticks=x_pos, labels=xtick_labels, fontsize=ticker_fontsize, rotation=xticker_rotate)
			plt.yticks(fontsize=ticker_fontsize, rotation=yticker_rotate, ticks=ytick_locations, labels=ytick_labels )

		self.set_title(title, fontsize=self.title_font)
		plt.tight_layout()
		if subplot is None:
			if outpath is not None: plt.savefig(outpath)
			if show: plt.show()

	def add_text(self, X, Y, textstr, α=0.05, β=0.95, xTextLoc=None, yTextLoc=None):
		if textstr is None: return
		mX = np.min(X)
		maX = np.max(X)

		mY = np.min(Y)
		maY = np.max(Y)

		if xTextLoc is None: xLoc = α*(maX - mX) + mX
		else: xLoc = xTextLoc

		if yTextLoc is None: yLoc = β*(maY - mY) + mY
		else: yLoc = yTextLoc

		props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
		plt.text(xLoc, yLoc, textstr, fontsize=14, verticalalignment='top', bbox=props)

	def set_title(self, title, fontsize=16):
		plt.title(title, fontsize=fontsize)


	def set_xlabel(self, xlabel, fontsize=16):
		plt.xlabel(xlabel, fontsize=fontsize)

	def set_ylabel(self, ylabel, fontsize=16):
		plt.ylabel(ylabel, fontsize=fontsize)

	def show(self, save_path=None, title=None, xlabel=None, ylabel=None, imgText=None, 
					xlim=None, ylim=None, xTextShift=0.05, yTextShift=0.95,
					xTextLoc=None, yTextLoc=None):

		if title is not None: self.set_title(title, fontsize=self.title_font)
		if xlabel is not None: self.set_xlabel(xlabel, fontsize=self.xfont)
		if ylabel is not None: self.set_ylabel(ylabel, fontsize=self.yfont)
		if imgText is not None: 
			self.add_text(X, Y, imgText, α=xTextShift, β=yTextShift, xTextLoc=xTextLoc, yTextLoc=yTextLoc)

		if save_path is None: plt.show()
		else: plt.savefig(save_path)





#!/usr/bin/env python

#import os
#import sys
#if os.path.exists('/home/chieh/code/wuML'):
#	sys.path.insert(0,'/home/chieh/code/wuML')
#import wuml
#%matplotlib notebook

from matplotlib import pyplot as plt
from mpl_toolkits import mplot3d
import matplotlib
import numpy as np


class mesh3d:
	"""A class that generates 3d mesh plots"""
	def __init__(self, X, Y, Z, title='title', xlabel='xlable', ylabel='ylable', zlabel='zlable', meshType='level', imgText=None, outpath=None, 
					subplot=None, xlim=None, ylim=None, xTextShift=0.05, yTextShift=0.95,
					ticker_fontsize=9, xTextLoc=None, yTextLoc=None, color='blue', marker=',', show=True, 
					xticker_rotate=0, yticker_rotate=0, zticker_rotate=0,
					xtick_locations=None, xtick_labels=None, ytick_locations=None, ztick_locations=None, ytick_labels=None, ztick_labels=None,
					title_font=13, xfont=10, yfont=10, zfont=10, figsize=None):
		"""calculates the probability that "token" is found in spam emails
		: meshType: 'level', 'scatter', 'wire'
		:param token: (str)
		:return: (float) probability "token" is spam based on training emails
		"""
		font = {'family' : 'normal', 'weight' : 'bold', 'size'   : title_font}
		matplotlib.rc('font', **font)

		self.title_font = title_font
		self.xfont = xfont
		self.yfont = yfont
		self.zfont = zfont
		self.already_called_plot_line = False

		if figsize is not None: plt.figure(figsize=figsize)

		fig = plt.figure()
		ax = plt.axes(projection='3d')

		if meshType == 'wire':
			ax.plot_wireframe(X, Y, Z, color='blue')
		elif meshType == 'level':
			ax.contour3D(X, Y, Z, 50, cmap='binary')
		elif meshType == 'scatter':
			ax.scatter3D(X, Y, Z, c=Z, cmap='Greens');
		elif meshType == 'surface':
			ax.plot_surface(X, Y, Z, rstride=1, cstride=1, cmap='viridis', edgecolor='none')
		else:
			raise

		ax.set_xlabel(xlabel, fontsize=self.xfont)
		ax.set_ylabel(ylabel, fontsize=self.yfont)
		ax.set_zlabel(zlabel, fontsize=self.zfont)
		ax.set_title(title, fontsize=self.title_font)

		plt.xticks(ticks=xtick_locations, labels=xtick_labels, fontsize=ticker_fontsize, rotation=xticker_rotate)
		plt.yticks(fontsize=ticker_fontsize, rotation=yticker_rotate, ticks=ytick_locations, labels=ytick_labels )
		plt.tick_params(axis='z', which='major', labelsize=ticker_fontsize, rotation=zticker_rotate)

		plt.tight_layout()
		plt.show()




	def plot_line(self, X, Y, title, xlabel, ylabel, imgText=None, outpath=None, 
					subplot=None, xlim=None, ylim=None, xTextShift=0.05, yTextShift=0.95,
					ticker_fontsize=9, xTextLoc=None, yTextLoc=None, color='blue', marker=',', show=True, 
					xticker_rotate=0, yticker_rotate=0,
					xtick_locations=None, xtick_labels=None, ytick_locations=None, ytick_labels=None):
		"""create a default 2D line plot
		
		: X: (float) the values for x-axis
		: Y: (float) the values for y-axis
		: xTextShift: (float) Uses the range of x values as total, sets a percentage of place the text
		: yTextShift: (float) Uses the range of y values as total, sets a percentage of place the text
		: xTextLoc: (float) Uses absolute value and overrides xTextShift
		: yTextLoc: (float) Uses absolute value and overrides yTextShift
		: marker types 		".": point		https://www.w3schools.com/python/matplotlib_markers.asp
							"o": circle
							",": just a pixel size
							"s": square
							"d": dimond
							"^": triangle
							"v": upside down triangle
							"+": plus
							"x": X
		:return: None
		"""

		if X is None: X = np.arange(1, len(Y)+1)
		if subplot is not None: plt.subplot(subplot)

		self.add_plot(X,Y, color, marker)

		#plt.tick_params(axis='both', which='both', labelsize=ticker_fontsize)
		plt.xticks(ticks=xtick_locations, labels=xtick_labels, fontsize=ticker_fontsize, rotation=xticker_rotate)
		plt.yticks(fontsize=ticker_fontsize, rotation=yticker_rotate, ticks=ytick_locations, labels=ytick_labels )

		self.set_title(title, fontsize=self.title_font)
		self.set_xlabel(xlabel, fontsize=self.xfont)
		self.set_ylabel(ylabel, fontsize=self.yfont)
		self.add_text(X, Y, imgText, α=xTextShift, β=yTextShift, xTextLoc=xTextLoc, yTextLoc=yTextLoc)

		if xlim is not None: plt.xlim(xlim)
		if ylim is not None: plt.ylim(ylim)
		
		plt.tight_layout()
		if subplot is None:
			if outpath is not None: plt.savefig(outpath)
			if show: plt.show()


	def add_text(self, X, Y, textstr, α=0.05, β=0.95, xTextLoc=None, yTextLoc=None):
		if textstr is None: return
		mX = np.min(X)
		mY = np.min(Y)

		if xTextLoc is None: xLoc = α*(np.max(X) - mX) + mX
		else: xLoc = xTextLoc

		if yTextLoc is None: yLoc = β*(np.max(Y) - mY) + mY
		else: yLoc = yTextLoc

		props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
		plt.text(xLoc, yLoc, textstr, fontsize=14, verticalalignment='top', bbox=props)



	def set_title(self, title, fontsize=16):
		plt.title(title, fontsize=fontsize)

	def set_xlabel(self, xlabel, fontsize=16):
		plt.xlabel(xlabel, fontsize=fontsize)

	def set_ylabel(self, ylabel, fontsize=16):
		plt.ylabel(ylabel, fontsize=fontsize)

	def add_plot(self, X,Y, color='blue', marker=','):
		#color='blue'        # specify color by name
		#color='g'           # short color code (rgbcmyk)
		#color='0.75'        # Grayscale between 0 and 1
		#color='#FFDD44'     # Hex code (RRGGBB from 00 to FF)
		#color=(1.0,0.2,0.3) # RGB tuple, values 0 to 1
		#color='chartreuse'; # all HTML color names supported

		plt.plot(X,Y, color=color, marker=marker)

	def show(self, save_path=None, title=None, xlabel=None, ylabel=None, imgText=None, 
					xlim=None, ylim=None, xTextShift=0.05, yTextShift=0.95,
					xTextLoc=None, yTextLoc=None):

		if title is not None: self.set_title(title, fontsize=self.title_font)
		if xlabel is not None: self.set_xlabel(xlabel, fontsize=self.xfont)
		if ylabel is not None: self.set_ylabel(ylabel, fontsize=self.yfont)
		if imgText is not None: 
			self.add_text(X, Y, imgText, α=xTextShift, β=yTextShift, xTextLoc=xTextLoc, yTextLoc=yTextLoc)

		plt.tight_layout()
		if save_path is None: plt.show()
		else: plt.savefig(save_path)


	def plot_line_with_error_area(x, y, error, show=False):
		#x = np.linspace(0, 30, 30)
		#y = np.sin(x/6*np.pi)
		#error = np.random.normal(0.1, 0.02, size=y.shape)
		#y += np.random.normal(0, 0.1, size=y.shape)
		
		plt.plot(x, y, 'k-')
		plt.fill_between(x, y-error, y+error)
		if show: plt.show()


if __name__ == "__main__":
#	# Example 1
#	def f(x, y):
#		return np.sin(np.sqrt(x ** 2 + y ** 2))
#
#	x = np.linspace(-6, 6, 30)
#	y = np.linspace(-6, 6, 30)
#	
#	X, Y = np.meshgrid(x, y)
#	Z = f(X, Y)
#
#	mesh3d(X, Y, Z, title='Set Title Here', xlabel='xlable', ylabel='ylable', zlabel='zlable', meshType='wire')
#	import pdb; pdb.set_trace()

	# Example 2
	def f(x, y):
		loss = 5* x**2 + 6*x*y - 16*x + 3*y**2 - 12*y + 14
		return loss

	x = np.linspace(-5, 5, 10)
	y = np.linspace(-5, 5, 10)
	
	X, Y = np.meshgrid(x, y)
	Z = f(X, Y)

	mesh3d(X, Y, Z, title='Error at different α,β', xlabel='α', ylabel='β', zlabel='error', meshType='wire')
	import pdb; pdb.set_trace()
